#include <stdio.h>
#include <memory.h>
#include <string.h>
#include <stdlib.h>

struct vector3f
{
	float x,y,z; 
};

struct vector4f
{
	float x,y,z,w; 
};

struct vector2f
{
	float tx,ty; 
};

struct or1_head
{
	unsigned int version;
	char name[16];
};

struct msh_bbox {
	struct vector3f		min;
	struct vector3f		max;
};

struct mdl_head
{
	unsigned int unk;
	unsigned int faces;
	unsigned int verts;
  int var[6];
	struct vector3f	mat3[4];
};

struct map_head
{
	unsigned int unk;
  struct msh_bbox bb;
	unsigned int unk2;
	unsigned int unk3;
  struct msh_bbox bb2;
  unsigned short us[5];
	unsigned char unk4;
  unsigned int var[1000];
};

struct face
{
	unsigned short f0,f1,f2; 
};

int main(int argc, char* argv[])
{
  or1_head hdr;
  mdl_head mdlhdr;
  map_head maphdr;
  unsigned long i;
	char fileName[256];
  FILE *inputFile;

	printf("Orion v1 MSH,MAP parser [201206xx]\n");
	if (argc < 2) 
  {
		printf("usage: parse_or1 < srcfile.msh > or < srcfile.map >\n");
		return 1;
	}

	strcpy((char*)fileName,argv[1]);

  inputFile = fopen(fileName,"rb");
	if(!inputFile)
  {
 	printf("ERR: cannot open.\n");
  return 1;
  }

	fseek(inputFile,0,SEEK_SET);

  fread( &hdr, sizeof(or1_head), 1, inputFile );
  printf("Version 0x%04X\nType '%s'\n",hdr.version,hdr.name);

	if (strstr(hdr.name,"Orion engine")) {
///////////////////////////////////////////////////////////////
  printf("*** Model file header:\n");
  fread( &mdlhdr, sizeof(mdl_head), 1, inputFile );
  printf("Unk 0x%08X (%d)\n",mdlhdr.unk,mdlhdr.unk);
  printf("Faces 0x%08X (%d)\n",mdlhdr.faces,mdlhdr.faces);
  printf("Verts 0x%08X (%d)\n",mdlhdr.verts,mdlhdr.verts);
  for (i=0;i<6;i++)
  printf("Var 0x%08X (%d) (%f)\n",mdlhdr.var[i],mdlhdr.var[i],*(float*)&mdlhdr.var[i]);
  for (i=0;i<4;i++)
  printf("Mat3[%u] %f %f %f\n",i,mdlhdr.mat3[i].x,mdlhdr.mat3[i].y,mdlhdr.mat3[i].z);

  vector3f * v = new vector3f[mdlhdr.verts];
  fread(v, sizeof(vector3f), mdlhdr.verts, inputFile );
//  for (i=0;i<mdlhdr.verts;i++)
//  printf("v %f %f %f\n",v[i].x,v[i].y,v[i].z);

  fread(v, sizeof(vector3f), mdlhdr.verts, inputFile );
//  for (i=0;i<mdlhdr.verts;i++)
//  printf("vn %f %f %f\n",v[i].x,v[i].y,v[i].z);
  delete [] v;

  face * f = new face[mdlhdr.faces];
  fread(f, sizeof(face), mdlhdr.faces, inputFile );
//  for (i=0;i<mdlhdr.faces;i++)
//  printf("f %d/%d/%d\n",f[i].f0,f[i].f1,f[i].f2);
  delete [] f;

  vector2f * vt = new vector2f[mdlhdr.verts];
  fread(vt, sizeof(vector2f), mdlhdr.verts, inputFile );
//  for (i=0;i<mdlhdr.verts;i++)
//  printf("vt %f %f\n",vt[i].tx,vt[i].ty);
  delete [] vt;
///////////////////////////////////////////////////////////////
	} else 
	if (strstr(hdr.name,"Orion Map")) {
///////////////////////////////////////////////////////////////
  printf("*** Map file header:\n");
  fread( &maphdr, sizeof(map_head), 1, inputFile );
  printf("Unk 0x%08X (%d)\n",maphdr.unk,maphdr.unk);
  printf("bb_min (%f %f %f) bb_max (%f %f %f)\n",
   maphdr.bb.min.x,maphdr.bb.min.y,maphdr.bb.min.z,maphdr.bb.max.x,maphdr.bb.max.y,maphdr.bb.max.z);
  printf("Unk2 0x%08X (%d)\n",maphdr.unk2,maphdr.unk2);
  printf("Unk3 0x%08X (%d)\n",maphdr.unk3,maphdr.unk3);
  printf("bb2_min (%f %f %f) bb2_max (%f %f %f)\n",
   maphdr.bb2.min.x,maphdr.bb2.min.y,maphdr.bb2.min.z,maphdr.bb2.max.x,maphdr.bb2.max.y,maphdr.bb2.max.z);
  for (i=0;i<5;i++)
  printf("U2Var 0x%04X (%d)\n",maphdr.us[i],maphdr.us[i]);
  printf("U1Var 0x%02X (%d)\n",maphdr.unk4,maphdr.unk4);

  for (i=0;i<1000;i++)
  printf("U4Var 0x%08X (%d) (%f)\n",maphdr.var[i],maphdr.var[i],(float)*(float*)&maphdr.var[i]);
///////////////////////////////////////////////////////////////
  } // map
	else 
  printf("UNKNOWN!\n");

	printf("pos is 0x%08X (%u)\n",ftell(inputFile),ftell(inputFile));
	//printf("Full size is 0x%08X (%u)\n",fullsize,fullsize);

  fclose(inputFile);
	return 0;
	}